using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Collections; // needed for Marshalling

namespace AlteryxAPISample2
{
	public partial class Form1 : Form
	{
		bool bfirstLoad = true; // only need to set up the column information once
		int columnCount = 0;	// keep track of the number of columns in my data

		public Form1()
		{
			InitializeComponent();

			// Initialize the date time picker 
			DateTime today = System.DateTime.Now;
			dateTimePicker1.MaxDate = today.AddDays(-1);
			dateTimePicker1.MinDate = today.AddDays(-365);  // back off 1 year

			// Load Locations into the Combobox
			LoadLocations();

			// Load the current Alteryx Version
			try
			{
				labelAlteryxVersion.Text = "Alteryx Version: " + Alteryx.AlteryxApi.AlteryxGetVersion();
			}
			catch (Alteryx.AlteryxApiException e)
			{
				MessageBox.Show(e.Message);
			}
		}

		// public class for storing my location data 
		public class locationData
		{
			private string displayLocation;
			private string weatherStation;
			public locationData(string strData, string strWS)
			{
				displayLocation = strData;
				weatherStation = strWS;
			}

			// to get the display location to the ComboBox
			public override string ToString()
			{
				return displayLocation;
			}

			public String Location { get { return displayLocation; } }
			public String WeatherStation { get { return weatherStation; } }
		}

		//An array for all of my Location Data
		private ArrayList m_nLocationData = new ArrayList();

		private void LoadLocations()
		{
			// this could be loaded with any amount of location data
			locationData loc1 = new locationData("Boulder, CO", "KCOBOULD16");
			locationData loc2 = new locationData("Estes Park, CO", "KCOESTES2");
			locationData loc3 = new locationData("Keystone, CO", "KCOKEYST1");
			m_nLocationData.Add(loc1);
			m_nLocationData.Add(loc2);
			m_nLocationData.Add(loc3);

			foreach (locationData ld in m_nLocationData)
				comboBox1.Items.Add(ld.ToString());
			comboBox1.SelectedIndex = 0;  // default to Boulder
		}

		// Add an new TextBox Column to my DataGridView
		private void AddNewTextBoxColumn(String strName, String strType, String strSource)
		{
			System.Windows.Forms.DataGridViewTextBoxColumn newDesc = new DataGridViewTextBoxColumn();
			newDesc.HeaderText = strName;
			newDesc.Name = strName + columnCount.ToString();
			newDesc.ReadOnly = true;
			newDesc.Resizable = DataGridViewTriState.True;
			newDesc.DisplayIndex = columnCount++;
			newDesc.ToolTipText = "Source:" + strSource;

			// this is where I can store information about my column that came from Alteryx
			newDesc.Tag = strType;
			this.dataGridView1.Columns.Add(newDesc);
		}

		// Add a new CheckBox Column to my DataGridView
		private void AddNewCheckBoxColumn(String strName, String strType, String strSource)
		{
			System.Windows.Forms.DataGridViewCheckBoxColumn newDesc = new DataGridViewCheckBoxColumn();
			newDesc.HeaderText = strName;
			newDesc.Name = strName + columnCount.ToString();
			newDesc.ReadOnly = true;
			newDesc.Resizable = DataGridViewTriState.True;
			newDesc.DisplayIndex = columnCount++;
			newDesc.ToolTipText = "Source:" + strSource;

			// this is where I can store information about my column that came from Alteryx
			newDesc.Tag = strType;
			this.dataGridView1.Columns.Add(newDesc);
		}
		
		//  Callback method for Alteryx - this is how I get my information into the DataGrid
		private int CallbackMessage(System.Int64 userData, System.Int32 nToolId, Alteryx.AlteryxApi.E_CallbackStatus nStatus, [MarshalAs(UnmanagedType.LPWStr)]string pMessage)
		{
			switch (nStatus)
			{
			case Alteryx.AlteryxApi.E_CallbackStatus.STATUS_Error:
				MessageBox.Show(pMessage);
				return 1;
			case Alteryx.AlteryxApi.E_CallbackStatus.STATUS_OutputRecord:
			{
				if (nToolId == 20)  // this is the DATA
				{
					// Loading up the data
					// 1=Hour 3=am/pm 5=temperature 7=dewpoint
					string[] words = pMessage.Split('\"');
					int j = 1;  // start at 1

					int nRow = this.dataGridView1.Rows.Count;
					dataGridView1.Rows.Add();
					for (int i = 0; i < columnCount; i++)
					{
						if (j < words.Length)
						{
							String strType = dataGridView1.Columns[i].Tag as String;
							if (strType == "Double")
							{
								// trim off temperature and dewpoint to nn.nn
								String strFormat = String.Format("{0:0.##}", System.Convert.ToDouble(words[j]));
								dataGridView1[i, nRow].Value = strFormat;
							}
							else
								dataGridView1[i, nRow].Value = words[j];
							j += 2;
						}
					}
				}
				else if (nToolId == 22 && bfirstLoad)  // this is the FieldInfo DATA
				{
					// A more dynamic way of figuring out your DataGridView
					// Guaranteed to be a defined size and look like:
					// Name, Type, Size, Scale, Source
					string[] words = pMessage.Split('\"');
					if (words[3] == "Bool")
					{
						// create a checkbox column - Name, Type and Source
						AddNewCheckBoxColumn(words[1], words[3], words[9]);
					}
					else
					{
						// create a TextBox Column - Name, Type and Source					
						AddNewTextBoxColumn(words[1], words[3], words[9]);
					}
				}
				break;
			}
			case Alteryx.AlteryxApi.E_CallbackStatus.STATUS_OutputFieldNames:
			{
				// Easier way if you already know your dataGridView column types
				// But maybe the Headers change
				if (nToolId == 20 && bfirstLoad)
				{
				    // \"Hour\",\"AM\",\"Temp\",\"Dewpoint\"
				//    string[] words = pMessage.Split('\"');
				//    dataGridView1.Columns[0].HeaderText = words[1];
				//    dataGridView1.Columns[1].HeaderText = words[3];
				//    dataGridView1.Columns[2].HeaderText = words[5];
				}
				break;
			}
			}
			return 0;
		}

		void AddValue(StringBuilder wizardValues, string strName, string strValue)
		{
			wizardValues.Append("\t<Value name=\"");
			wizardValues.Append(strName);
			wizardValues.Append("\">");
			wizardValues.Append(strValue);
			wizardValues.Append("</Value>\n");
		}

        private void GetTemperatures(int index)
        {
			// make sure you have a valid selection 
			if (! (0 <= index && index < m_nLocationData.Count))
				return;

 			DateTime dt = dateTimePicker1.Value;
			locationData loc = (locationData)m_nLocationData[index];

			// Load up the WizardValues for Alteryx
			StringBuilder wizardValues = new StringBuilder();
			wizardValues.Append("<WizardValues>\n");
			AddValue(wizardValues, "Location", loc.WeatherStation);
			AddValue(wizardValues, "Date", String.Format("{0:yyyy-MM-dd}", dt));
            wizardValues.Append("</WizardValues>\n");
           
            //MessageBox.Show(wizardValues.ToString());

            try
            {
                Alteryx.AlteryxApi.CallbackMessage pCallbackMessage = new Alteryx.AlteryxApi.CallbackMessage(this.CallbackMessage);
                String strPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
				String strWizard = "..\\..\\AlteryxAPISample2.yxwz";

                Alteryx.AlteryxApi.AlteryxRunWizard(System.IO.Path.Combine(strPath, strWizard),
                    wizardValues.ToString(),
                    null,
                    null,
                    pCallbackMessage,
                    0);
            }
            catch (Alteryx.AlteryxApiException e)
            {
                MessageBox.Show(e.Message);
            }
        }

 		private void LoadData()
		{
			// clear out current data
			dataGridView1.Rows.Clear();

			GetTemperatures(comboBox1.SelectedIndex);
			if (dataGridView1.Rows.Count != 0)
			{
				dataGridView1.ColumnHeadersVisible = true;
				dataGridView1.AutoResizeColumns();
			}
			bfirstLoad = false;
		}

		private void button1_Click(object sender, EventArgs e)
		{
			LoadData();
		}
	}
}